import { HIGHLIGHTED_COLOR, formatTimestamp } from '@/analyst/utilities'
import type { ButtonProps } from '@blueprintjs/core'
import { Button, ButtonGroup, Card } from '@blueprintjs/core'
import { useHost } from '@inject/graphql/connection/host'
import type { Exercise } from '@inject/graphql/fragments/Exercise.generated'
import { downloadLogUrl } from '@inject/shared/config'
import { useMemo, type FC, type ReactNode } from 'react'

type SelectingDetails = {
  onSelect: () => void
}

type ManagingDetails = {
  startButtonProps: ButtonProps
  removeButtonProps: ButtonProps
  pauseButtonProps: ButtonProps
}

type ExerciseCardProps = {
  isSelected?: boolean
  exercise: Exercise
} & (
  | {
      type: 'selecting'
      details: SelectingDetails
    }
  | { type: 'managing'; details: ManagingDetails }
)

const ExerciseCard: FC<ExerciseCardProps> = ({
  exercise,
  isSelected,
  type,
  details,
}) => {
  const host = useHost()

  const buttons: ReactNode = useMemo(
    () =>
      type === 'selecting' ? (
        !isSelected && (
          <Button onClick={details.onSelect} icon='select'>
            Select
          </Button>
        )
      ) : (
        <>
          {!exercise.running && !exercise.finished && (
            <Button icon='play' {...details.startButtonProps}>
              {exercise.exerciseStart ? 'Resume' : 'Start'}
            </Button>
          )}
          {exercise.running && (
            <Button icon='pause' {...details.pauseButtonProps}>
              Pause
            </Button>
          )}
          {!exercise.running && (
            <Button icon='cross' {...details.removeButtonProps}>
              Remove
            </Button>
          )}
          {exercise.exerciseStart && (
            <a
              href={downloadLogUrl(host || '', exercise.id)}
              target='_blank'
              rel='noreferrer'
            >
              <Button icon='download'>Download logs</Button>
            </a>
          )}
        </>
      ),
    [
      details,
      exercise.exerciseStart,
      exercise.finished,
      exercise.id,
      exercise.running,
      host,
      isSelected,
      type,
    ]
  )

  return (
    <Card
      style={{
        backgroundColor: isSelected ? HIGHLIGHTED_COLOR : undefined,
        display: 'flex',
        flexDirection: 'column',
        justifyContent: 'space-between',
      }}
    >
      <div>
        <h3>{`Exercise: ${exercise.name}`}</h3>
        <h4>{`Definition: ${exercise.definition?.name || exercise.definition?.id}`}</h4>
        <p>
          {`${exercise.running ? 'Running' : 'Not running'}, \
                        ${exercise.finished ? 'Finished' : 'Not finished'}`}
        </p>
        <span style={{ fontWeight: 'bold' }}>Team count: </span>
        <span>{exercise.teams.length}</span>
        <br />
        <span style={{ fontWeight: 'bold' }}>Start: </span>
        <span>
          {exercise.exerciseStart
            ? formatTimestamp(exercise.exerciseStart)
            : 'not started yet'}
        </span>
      </div>

      <div style={{ marginTop: '1rem', alignSelf: 'end' }}>
        <ButtonGroup vertical alignText='left'>
          {buttons}
        </ButtonGroup>
      </div>
    </Card>
  )
}
export default ExerciseCard
